#!/bin/bash
######################################################################
#File Name: run-psc-vs-rfi-win.sh
#File Type: Bash-Script
#Author: Benjamin Peter
#Creation: 20.08.2009 11:03:16
#Last Update: 4.12.2009 
#Description: this script implements the procedure described to 
#  distinguish between an island model and a model of 
#  population size change, as presented in Peter & Excoffier (2009)
######################################################################
#Essential parameters: 
######################################################################
#The following section contains the prior distributions for the parameters 
# of the island model and the model of population size change. In Bayesian statistics prior distributions represent the a priori degree of belief in a given parameter. Common choices may be based on results from previous studies. If little is known about the study system, wide priors may be accurate.
#
##Prior distributions for parameters are defined in the following way:
#<ParameterName>="<Shape of Distribution> <One or more parameters for distribution>"
#note the equality sign and the quotation mark in the above definition

##Shapes of the distribution are passed to ABCtoolbox (Wegmann et al. (2009)) where currently the following distributions are implemented:
#constant: If a parameter should be fixed at a given value a, this can be set with "constant a"
#unif: Uniform distribution X ~[a, b]. This distribution is invoked with "unif a b"
#logunif: Log-uniform distribution, corresponding to the uniform distribution on a logarithmic scale: log(X)~[log(a), log(b)]. It is invoked with "logunif a b"
#norm: a truncated normal distribution with X~N[mu, sigma], where mu and Sigma are the mean and standard deviation of the distribution, truncated such that a<=x<=b. It is invoked with "norm mu sigma a b"
#lognorm: A log-normal distribution such that log(X)~N[mu, sigma], with mu and sigma chosen so that the resulting distribution has mean mu and standard deviation sigma on the natural scale. This distribution is truncated such taht a<=X<=b. It is invoked using "lognorm mu sigma a b"

######################################################################
##INPUT 
#Currently, we only support microsatellite data. As input, the path to an file containing the observed microsatellites is required. The file has to be in arlequin file format. If your data is stored in a different format, we recommend to use PGDSpider (Lischer & Excoffier, 2009) for conversion.

#add path to your data file
inputFilePath="./EXAMPLE.arp"
#give name of output file
output="output.txt"
#add path to a log file
logfile="logfile.log"
#add a prefix for all files created
filename="psc-vs-rfi"
#this script creates several files, set to true if you want to delete them automatically.
tidyUp=false
#if set to -1, these quantities is estimated automatically. Procedure used is quite crude and slow and might fail if the number of loci is very low (<5), in this case set the number of Loci manually below
noLoci=-1
sampleSize=-1

######################################################################
##MUTATION MODEL
#The mutation model is set to be the same for both models. As we are using simcoal(Excoffier et al, 2005) for coalescence simulation, we use the stepwise mutation model implemented there. We assume that the mutation rate varies over loci, with the variation being gamma distributed. Parameters for the mutation model are the mean mutation rate, mu, and the shape parameter alpha for the shape of the gamma distribution:
mu="unif  0.0001  0.0005"
alpha="unif  2  20"

######################################################################
##MODEL OF POPULATION SIZE CHANGE PARAMETERS
# for the model model of population size change, we set priors on the current effective population size, N0, the relative ancient population size a, and the time since the onset of the decline t. For a graphical representation of this model see Figure 1b of Peter & Excoffier (2009)
n0="logunif  1000  10000"
a="unif  -3  3"
t="logunif  1  1000"

######################################################################
##RECENT FINITE ISLAND MODEL PARAMETERS
# for the island model, we set priors on the number of demes D, the (haploid) deme size N, the Migration parameter Nm, the time since a population split T, and the population size before this split NA. Note that if you'd like to not implement a population split, you may set T to a very large number. For a graphical representation of this model see Figure 1a of Peter & Excoffier (2009)
D="unif 100 100"
N="unif 200 200"
Nm="logunif 0.1 100"
T="logunif 100 100000"
Na="logunif 1000 50000"
S="unif 0 1"

######################################################################
##ABC SETTINGS
#these are settings for the ABC procedure. 

#Unfortunately, the simulations required for ABC can take several days on a single CPU. However, paralellization on a grid can drastically improve this time
#if you have a multicore processor, the numberOfCPUs parameter can divide the most computationally intensive part on various cpus to let it run faster
numberOfCPUs=2
#

#total number of simulations. The larger the better, we suggest a value of 10^5 - 10^6
#this parameter is the most important one in defining the runtime of the procedure
numberOfIterationsPerModel=200

#number of simulations to retain in the regression step
numberOfRetainedSimulations=10

#how to transform the data, passed to abcEst. See its documentation for details
#0: no transformation; 1: log-transformation 2: log-tan-transformation
transformation=2

######################################################################
##MODEL CHOICE
#select which method you'd like to use (or both)
#beaumont uses the model choice procedure by Beaumont (2008)
#  note that this script requires the VGAM package for R
#pritchard uses the model choice procedure by Pritchard et al (2000)
beaumont=true
pritchard=true

######################################################################
##Summary statistics to use
#Enter true for the statistics you want to use
K=true
sdK=true
H=true
sdH=true
GW=true
sdGW=false
R=true
sdR=false
FIS=true
######################################################################
##PROGRAMS TO USE
#Set these to the correct pathes to the programs used
abcSampler="ABCSampler"
coalescenceSimulator="Simcoal3_BC6"
ssCalculator="arlsumstat"
estimator="abcEst3"
rPath="c:/Program Files/R/R-2.10.0/bin/R"

#If you use ABCSampler, please cite
#Wegmann D, Leuenberger C, Neuenschwander S & Excoffier L (2009:
#ABCtoolbox: A versatile toolkit for Approximate Bayesian Computation
#Bioinformatics, submitted

#If you use Simcoal, please cite
#Excoffier L, Laval G & Schneider S (2005): Arlequin ver 3.0: An 
#integrated software package for population genetic data analysis
#Evolutionary Bioinformatics Online, 1, 47-50

#If you use arlsumstat and abcEst, please cite
#Laval G & Excoffier L (2005): Simcoal 2.0: a program to simulate genomic
#diversity over large recombining regions in a subdivided population with 
#complex history, Bioinformatics 20, 2485-2487

#If you use R, please cite
#R Development Core Team (2009). R: A language and environment for 
#statistical computing. R Foundation for Statistical Computing, 
#Vienna, Austria. ISBN 3-900051-07-0, URL http://www.R-project.org.


######################################################################
#in principle you should not have to change anything below here
######################################################################
#check for missing files
function checkMissing {
  if [ ! -f $1 ]; then
    echo "$1 not found; exiting";
    exit 1;
  fi
}

#copy files into working directory
cp apps/* .
cp scripts/* .

#check Input file
checkMissing $inputFilePath

#additional files for arlequin
checkMissing "ArlequinIniMake.sh"
checkMissing "arl_run.ars"

#files to make par files
checkMissing "make_botParFile.sh"
checkMissing "make_islParFile.sh"

#check executables
checkMissing $abcSampler
checkMissing $coalescenceSimulator
checkMissing $ssCalculator
checkMissing $estimator

#check execution scripts
checkMissing prepareObsFile.sh
checkMissing writeIslandEstFile.sh 
checkMissing writeInputFile.sh 
checkMissing runSims.sh 
checkMissing writeAbcEstFile.sh 
checkMissing runEstimator.sh 
checkMissing runModelChoice.sh
if [ $tidyUp == true ]; then
  checkMissing tidyUp.sh
  chmod +x tidyUp.sh
fi
chmod +x prepareObsFile.sh writeIslandEstFile.sh writeInputFile.sh runSims.sh writeAbcEstFile.sh runEstimator.sh runModelChoice.sh 

#check model Choice scripts & VGAM library
if [ $beaumont == true ]; then
  checkMissing "modelChoice_Beaumont.r"
  checkMissing "modelChoice_Beaumont_2.r"
  echo 'a<-require(VGAM);
  if (!a) print("ERROR: VGAM Package required for Model Choice Procedure by Beaumont")'|"$rPath" --no-save --slave
fi
if [ $pritchard == true ]; then
  checkMissing "modelChoice_Pritchard.r"
fi

echo "All required files found"

######################################################################
#estimate Number of Loci if not given
if [ $noLoci -eq -1 ]; then
  maxWords=0
  for i in  $(seq `wc -l <$inputFilePath`); do
    l=`head -n${i} $inputFilePath |tail -n1`;
    currLineWords=`echo $l |cut -d# -f1 | cut -d\" -f1 | wc -w`;
    if [ $maxWords -lt $currLineWords ] ; then
      maxWords=$currLineWords;
      echo $maxWords
    fi;
  done
fi;
noLoci=$((maxWords-2))
echo "Number of Loci inferred: $noLoci"

#estimate Sample Size
#do this by extracting the "SampleSize" tag from Arlequinfiles. Then, put everything on one line using the while loop
#and sum stdin with perl
if [ $sampleSize -eq -1 ]; then
  sumStr=`grep SampleSize $inputFilePath | cut -f2 -d= | while read line; do echo -n "$line "; done | sed 's/ /+/g' | sed 's/+$//'`
  sampleSize=$(($sumStr))
fi;
echo "Sample Size inferred: $sampleSize"
echo "######################################################################"


. prepareObsFile.sh $filename $inputFilePath $K $sdK $H $sdH $GW $sdGW $R $sdR $FIS
noStats=$?
echo "noStats: $noStats"
######################################################################
##start Simulations: first for bottleneck model
echo "######################################################################"
echo "start sims: bottleneck"
echo "######################################################################"
./writeBneckEstFile.sh $filename "$n0" "$a" "$t" "$mu" "$alpha" "$noLoci" "$sampleSize" 
./writeInputFile.sh $filename bot $numberOfIterationsPerModel $numberOfCPUs $coalescenceSimulator $ssCalculator
./runSims.sh $filename bot $abcSampler $coalescenceSimulator $ssCalculator $numberOfIterationsPerModel $numberOfCPUs $logfile $tidyUp
./writeAbcEstFile.sh $filename bot $estimator $noStats $numberOfRetainedSimulations $numberOfIterationsPerModel
./runEstimator.sh $filename bot $estimator
######################################################################
##now, repeat everything with island Model
echo  ... finished
echo "######################################################################"
echo
echo
echo

echo "######################################################################"
echo "start sims: island" 
echo "######################################################################"
./writeIslandEstFile.sh $filename "$D" "$N" "$T" "$Na" "$S" "$Nm" "$mu" "$alpha" "$noLoci" "$sampleSize"
./writeInputFile.sh $filename isl $numberOfIterationsPerModel $numberOfCPUs $coalescenceSimulator $ssCalculator
./runSims.sh $filename isl $abcSampler $coalescenceSimulator $ssCalculator $numberOfIterationsPerModel $numberOfCPUs $logfile 
./writeAbcEstFile.sh $filename isl $estimator $noStats $numberOfRetainedSimulations $numberOfIterationsPerModel
./runEstimator.sh $filename isl $estimator
echo  ... finished
echo "######################################################################"
echo
echo
echo
############################################################
#NOW USE MODEL CHOICE PROCEDURES
./runModelChoice.sh $filename "$rPath" $beaumont $pritchard $output



#FINALLY, TIDY UP
if [ $tidyUp == true ]; then
  ./tidyUp.sh $filename $inputFilePath
  rm tidyUp.sh
fi
