#author:Nicolas Ray
#-----------------------------------------------------------------------------------------------------------------------------
call_calmod <- function(input_list, input_obs_sumstat, input_obs_sumstat_as_file, best_sim_header, print_results=T,numsim=0)
{
# This function loads and formats the appropriate input files and call Mark's function ('calmod', below)
# This function assumes that parameters and summary stats are found in a big file (as output
# by simCoalABC or SPLATCHE), with one such file per different model
#
#Parameters (6)
#----------
# input_list is a vector containing the file names of the summary stats matrices.
#	  There are as many matrices as there are different models to compare
#	  Each matrix must contain a one-line header with the names of the sumstats
#	  to construct this list, e.g.: input<-c("instgrowth50_1mio_test.out", "slowgrowth_1mio_test.out")
# input_obs_sumstat is the filename of the file containing the observed summary stats.
#	  This file must also have a header
# input_obs_sumstat_as_file is a boolean that controls if the observed summary stats are given in a file (T) or are passed as a
#   vector (F) to the function
# best_sim_header is a boolean that indicates wether there is a header in the bast sim files
# print_results is a boolean that indicates wether the results are printed or just returned (to be passed to another function)
# numsim is the number of simuls. If known, it improves the speed of reading. If not (i.e., =0), it's slower...
#
#Working directory: it must be set directly through the command "change working directory" in the R console
#
#How to call, e.g.:
#	source("C:/Projets/ABC_bottleneck/R/calcmod_commented_v2.r"); call_calmod(four_models_input, "temp_HUM_CONC2.sum", T, T, T, 5000)

#Requirements
#------------
#	VGAM package (for vglm function). It can be installed from:
#	http://www.stat.auckland.ac.nz/~yee/VGAM/download.shtml

#Outputs
#------------
#	The script outputs the relative probability of each model. They are found under $x2 in the output,
#	with each mu1, mu2, mu3, etc corresponding to a model, in the same order than they were entered in the input
#	model list (input_list)

#Notes
#----------
#The number of sum stat must be equal in each file, while the number of parameters can vary (and are not used anyway)
#The script can either be launched on the full simulation matrice and it computes the distances
#	to keep the given proportion of retained simulations, or (faster) it can be applied directly on
#	the, say, 5000 best simulations by setting the proporation of retained to 1.
#The proporation of retained sims must be set directly through the tolerance parameter 'tol'

#load required package VGAM
if(!library(VGAM, logical.return=TRUE)){
   print("The package VGAM needs to be installed in order for this script to work.")
   print("Script terminated.")
   return()
}

#error handling
if (length(input_list)<2){
   print("You need at least two matrices (files) in the input file vector to run this script!")
   print("Script terminated.")
   return()
}

#read target, the input observed sumstat file, and get the number of sumstat (=length of first line)
if (input_obs_sumstat_as_file==T){
  targetFrame<-read.table(input_obs_sumstat, header=T, comment.char="#")
  target<-data.matrix(targetFrame)
} else {
  #stats are passed directly as a vector
  target<-data.matrix(input_obs_sumstat)
}

numsumstat<-length(target[1,])

#read first matrix. If numsim=0 (default), it means we don't know the number of simul
#or it means there are a different number of simuls per model. In that case, the reading is
#very slow. We improve the reading speed is the number if simul is given
if (numsim==0){
   m_temp_frame<-read.table(input_list[1], header=best_sim_header, colClasses = "numeric", comment.char="")
   m_temp<-data.matrix(m_temp_frame)
   rm(m_temp_frame)
} else {
   print("Reading matrice no 1...")
   if(best_sim_header){
    m_temp_frame<-read.table(input_list[1], header=T, colClasses = "numeric", nrows=numsim+1, comment.char="")
   } else {
    m_temp_frame<-read.table(input_list[1], header=F, colClasses = "numeric", nrows=numsim, comment.char="")
   }
   m_temp<-data.matrix(m_temp_frame)
   rm(m_temp_frame)

   #alternatively
   #print("Reading matrice no 1...")
   #m_temp_frame<-scan(file=input_list[1], what="numeric", nlines=numsim+1)
   #m_temp<-data.matrix(m_temp_frame)
}


#keep only the sumstats columns and store them in m
num_tot<-length(m_temp[1,])
m<-m_temp[,c((num_tot-numsumstat+1):num_tot)]

#error handling
if (length(m[1,])!=length(target)){
   print("The number of summary stat is not equal in the observed and the simulated files!")
   print("Script terminated.")
   return()
}

#fill the model id vect (x in Beaumont function below) with "1"
x<-rep(1,length(m[,1]))

#treat the following matrices (models)
for(i in 2:length(input_list)){

	#read the file
	if (numsim==0){
		  m_temp_frame<-read.table(input_list[i], header=best_sim_header, comment.char="#")
   		m_temp<-data.matrix(m_temp_frame)
		  rm(m_temp_frame)
	} else {
   		print("Reading other matrice")
      if(best_sim_header){
   		 m_temp_frame<-read.table(input_list[i], header=T, colClasses = "numeric", nrows=numsim+1,comment.char="")
   		} else {
   		 m_temp_frame<-read.table(input_list[i], header=F, colClasses = "numeric", nrows=numsim,comment.char="")
   		}
   		m_temp<-data.matrix(m_temp_frame)
		  rm(m_temp_frame)

		  #alternatively
   		#print("Reading matrice no "+i+"...")
   		#m_temp_frame<-scan(file=input_list[i], what="numeric", nlines=numsim+1)
   		#m_temp<-data.matrix(m_temp_frame)
	}

	#keep only the sumstats columns and bind it to the main inputMatrices
	num_tot<-length(m_temp[1,])
	m<-rbind(m, m_temp[,c((num_tot-numsumstat+1):num_tot)])

	#fill the model id vect (x in Beaumont function below) with i values
	x<-c(x,rep(i,length(m_temp[,1])))
	rm(m_temp)
}

#transform the data frame into a matrix
sumstatMatrix<-data.matrix(m)

#fill the gwt vector, required in Beaumont's function.
#I build it as a vector of TRUE value of size=numsumstat
gwt <- rep(T,length(sumstatMatrix[1,]))

#set the tolerance, required in Beaumont's function
#if set to 1.0, it will use all simulations, in the case the input matrices are
#already the retained sims
tol <- 1.0

#call Beaumont's function, and print results if necessary
results<-calmod(target, x, sumstatMatrix, tol, gwt, F)
if (print_results) print(results)
return(results)

}
#-----------------------------------------------------------------------------------------------------------------------------
call_calmod_from_phaentus_files<-function(filename, observed_filename, doScaling=F, doWeightening=F){
   #this function calls callmod from a file which already containes the stats for all models 
   #and a first column with the names of the models. The working directory is asumed to contain the file
   
   #load required package VGAM
   if(!library(VGAM, logical.return=TRUE)){
      print("The package VGAM needs to be installed in order for this script to work.")
      print("Script terminated.")
      return()
   }
   
   #read observed file
   target<-data.matrix(read.table(observed_filename, header=T, nrow=1));
   #read sumstat file
   sumstats<-read.table(filename, header=T);
   #check if the same number of stats are there
   if(length(sumstats)!=(length(target)+1)){
      print ("The number of stat columns in the sum stat file does not correspond to the number of columns in the observed file!");
      return ();
   }
   #get first column and prepare the x-vector for Beaumont's script   
   x_names<-unique(sumstats[,1])
   x<-rep(1,length(sumstats[,1]));
   for(i in 2:length(x_names)){ x[sumstats[,1]==x_names[i]]<-i; }
   
   #set the tolerance, required in Beaumont's function
   #if set to 1.0, it will use all simulations, this is what we want 
   #as the file only containes the retained simulations
   tol <- 1.0

   #get sumstats as matrix NICO: inverted with next block
   sumstatMatrix<-data.matrix(sumstats[,2:length(sumstats)]);
   rm(sumstats);

 
   #fill the gwt vector, required in Beaumont's function.
   #I build it as a vector of TRUE value of size=numsumstat
   gwt <- rep(T,length(sumstatMatrix[1,]))
   
   
   
   #call Beaumonts function   
   results<-calmod(target, x, sumstatMatrix, tol, gwt, F, doScaling, doWeightening);
   return(results);
}

#-----------------------------------------------------------------------------------------------------------------------------
calmod <- function(target,x,sumstat,tol,gwt,rejmethod=T, doScaling=T, doWeightening=T)
{
# Original version from Mark Beaumont
#
# This function uses categorical regression to estimate the posterior probability of a particular model
#      under the ABC framework P(Y=y | S = s)
#
# target is the set of target summary stats - i.e. s, what you measured from the data.
# x is the parameter vector, Y (long vector of numbers from the simulations) and is the dependent variable for the regression
#          This is a categorical variable, taking values 1, 2, .. n where there are n categories (models)
# sumstat is an array of simulated summary stats, S (i.e. independent variables).
# tol is the required proportion of points nearest the target values
# gwt is a vector with T/F weights, weighting out any 'bad' values (determined by the simulation program - i.e. nan's etc)
# if rejmethod=T it doesn't bother with the regression, and just does rejection.


# If rejmethod=F it returns a list with the following components:-

# $x1 expected value on a logit scale, with standard errors of the estimate
# $x2 expected value on a natural scale - i.e. p(Y=y | S = s) This is what we would normally report.
# $vals - the Ys in the rejection region
# $wt - the Epanechnikov weight.
####NBB This should have been used in the regression, but appears not to have been in the original version. See
####     the comment below.
# $ss - the sumstats corresponding to the points in $vals. You could try weighting $vals and making a table.
#        the unweighted table would lead to Arnaud's 'posterior weights' for the different models.

if(missing(gwt))gwt <- rep(T,length(sumstat[,1]))

nss <- length(sumstat[1,][gwt])

# scale everything
scaled.sumstat <- sumstat
target.s <- target

#scaling:
if(doScaling){
  
    for(j in 1:nss){
    	scaled.sumstat[,j] <- normalise(sumstat[,j],sumstat[,j][gwt])
    }

    
    for(j in 1:nss){  
    	target.s[j] <- normalise(target[j],sumstat[,j][gwt])
    }
}
# calc euclidean distance

    sum1 <- rep(0,length(scaled.sumstat[,1]))
    for(j in 1:nss){
    	sum1 <- sum1 + (scaled.sumstat[,j]-target.s[j])^2
   }
   dst <- sqrt(sum1)
       #print(scaled.sumstat[1,1] )
       #print(target.s[1])
# includes the effect of gwt in the tolerance
    dst[!gwt] <- floor(max(dst[gwt])+10)

# wt1 defines the region we're interested in
    abstol <- quantile(dst,tol)   #if it does not work here, it means input sumstats are not in a matrix form!

    wt1 <- dst < abstol
    nit <- sum(wt1)

    if(rejmethod){
        l1 <- list(x=x[wt1],wt=0)
    }
    else{
        #calculate weighting
        if(doWeightening){
           regwt <- 1-dst[wt1]^2/abstol^2
        }

        catx <- as.numeric(names(table(x[wt1])))
        ncat <- length(catx)
        #yy: gives "true" model for each entry
        yy <- matrix(nrow=nit,ncol=ncat)
        for(j in 1:ncat){
        	yy[,j] <- as.numeric(x[wt1] == catx[j])
        }

        tr <- list()

        for(j in 1:nss){
        	tr[[j]] <- scaled.sumstat[wt1,j]
        }

        xvar.names <- paste("v",as.character(c(1:nss)),sep="")

        names(tr) <- xvar.names

          #fmla should look like yy(binary response, i.e. model) ~ xvar1 + xvar2 + xvar 3
               #meaning no interaction, with (independent) summary stats!
        fmla <- as.formula(paste("yy ~ ", paste(xvar.names, collapse= "+")))

        if(!doWeightening){
           fit1 <- vglm(fmla,data=tr,multinomial, qr.arg=T)
        } else {
           fit1 <- vglm(fmla,data=tr,multinomial,weights=regwt) 
           #comment to Laurent: the version described in the
           #manuscript appears not to have used the Epanechnikov weights.
            #I think this is a mistake, but since this is therefore untested, the 'corrected' version is left
           #commented here, but you might want to comment out the first one, and uncomment this, and see what
           #happens.
        }
        

        target.df <- list()
        for(j in 1:nss){
        	target.df[[j]] <- target.s[j]
        }
        names(target.df) <- xvar.names

        prediction1 <- predict.vglm(fit1,target.df,se.fit=T)
        prediction2 <- predict.vglm(fit1,target.df,type="response")

        if(!doWeightening){
          l1 <- list(x1=prediction1,x2=prediction2,vals=x[wt1],ss=sumstat[wt1,])
        }
        else {
          l1 <- list(x1=prediction1,x2=prediction2,vals=x[wt1],wt=regwt,ss=sumstat[wt1,])        
        }

    }

  #NICOLAS: return only the expected values on a natural scale (x2)
	#and not the full list of results
  return(l1["x2"])

}

#author: Mark Beaumont
normalise <- function(x,y){
  if(var(y) == 0)return (x - mean(y))
  (x-(mean(y)))/sqrt(var(y))
}


